@@ -32,12 +32,12 @@ private Q_SLOTS:
         T("Dec off");
         T("Dec 25 off");
         T("Dec 25-26 off");
-        T2("Dec 24-26,31 off", "Dec 24-26,Dec 31 off");
-        T2("Jan 1,6 off", "Jan 01,Jan 06 off");
-        T2("Dec 24,25,26", "Dec 24,Dec 25,Dec 26");
-        T2("Jan 03,Dec 04,24 off", "Jan 03,Dec 04,Dec 24 off");
-        T2("Jan 03, Dec 04, 24 off", "Jan 03,Dec 04,Dec 24 off");
-        T2("07:30-20:00; Jan 03,13,23,Dec 04,14,24 off", "07:30-20:00; Jan 03,Jan 13,Jan 23,Dec 04,Dec 14,Dec 24 off");
+        T("Dec 24-26,31 off");
+        T2("Jan 1,6 off", "Jan 01,06 off");
+        T("Dec 24,25,26");
+        T2("Jan 03,Dec 04,24 off", "Jan 03,Dec 04,24 off");
+        T2("Jan 03, Dec 04, 24 off", "Jan 03,Dec 04,24 off");
+        T2("07:30-20:00; Jan 03,13,23,Dec 04,14,24 off", "07:30-20:00; Jan 03,13,23,Dec 04,14,24 off");
         T("Dec 08:00");
         T("Dec 08:00-14:00");
         T("easter off");
@@ -95,6 +95,12 @@ private Q_SLOTS:
         T("Mar Su[1]-Oct Su[1]: 11:00-20:00; PH 11:00-20:00");
         T2("Mo 20:00-26:00", "Mo 20:00-26:00"); // https://github.com/osm-fr/osmose-backend/issues/1344
 
+        // https://bugs.kde.org/show_bug.cgi?id=445963
+        T("Th[1-2] 09:30-11:45");
+        T("Th[1,2] 09:30-11:45");
+        T("Mo[1,3,-1]"); // order as desired, -1 means last
+        T("Mo[1,3,2]"); // currently not normalized
+
         // from https://wiki.openstreetmap.org/wiki/Key:opening_hours#Simple_examples
         T("Mo-Fr 08:00-17:30");
         T("Mo-Fr 08:00-12:00,13:00-17:30");
@@ -156,7 +162,7 @@ private Q_SLOTS:
         T("2012 easter -2 days-2012 easter +2 days: open \"Around easter\"; PH off");
         T("24/7 closed \"always closed\"");
         T2("2013,2015,2050-2053,2055/2,2020-2029/3,2060+ Jan 1", "2013,2015,2050-2053,2055/2,2020-2029/3,2060+ Jan 01");
-        T("Jan 23-Feb 11,Feb 12 00:00-24:00; PH off");
+        T2("Jan 23-Feb 11,Feb 12 00:00-24:00; PH off", "Jan 23-Feb 11,Feb 12 00:00-24:00; PH off");
         T("Apr-Oct Su[2] 14:00-18:00; Aug Su[-1] -1 day 10:00-18:00; Aug Su[-1] 10:00-18:00; PH off");
         T("Mo-Fr 08:00-12:00, We 14:00-18:00; Su,PH off"); // open We morning too
         T("Mo-Fr 08:00-12:00; We 14:00-18:00; Su,PH off"); // closed We morning
@@ -181,8 +187,9 @@ private Q_SLOTS:
         T("Oct: We[1]");
 
         // from https://github.com/dfaure/DataNovaImportScripts/blob/master/saved_opening_hours
-        T3("Mo-Tu,Th-Fr 09:30-12:00; 2020 Dec 28 off; 2020 Dec 22,2020 Dec 29 off; We 15:00-17:00; 2020 Dec 23,2020 Dec 30 off; 2020 Dec 24,2020 Dec 31 off; Sa 10:00-12:00; 2020 Dec 26,2021 Jan 02 off; PH off", nullptr,
-           "Mo,Tu,Th,Fr 09:30-12:00; 2020 Dec 28 off; 2020 Dec 22,2020 Dec 29 off; We 15:00-17:00; 2020 Dec 23,2020 Dec 30 off; 2020 Dec 24,2020 Dec 31 off; Sa 10:00-12:00; 2020 Dec 26,2021 Jan 02 off; PH off");
+        T3("Mo-Tu,Th-Fr 09:30-12:00; 2020 Dec 28 off; 2020 Dec 22,2020 Dec 29 off; We 15:00-17:00; 2020 Dec 23,2020 Dec 30 off; 2020 Dec 24,2020 Dec 31 off; Sa 10:00-12:00; 2020 Dec 26,2021 Jan 02 off; PH off",
+           "Mo-Tu,Th-Fr 09:30-12:00; 2020 Dec 28 off; 2020 Dec 22,29 off; We 15:00-17:00; 2020 Dec 23,30 off; 2020 Dec 24,31 off; Sa 10:00-12:00; 2020 Dec 26,2021 Jan 02 off; PH off",
+           "Mo,Tu,Th,Fr 09:30-12:00; 2020 Dec 28 off; 2020 Dec 22,29 off; We 15:00-17:00; 2020 Dec 23,30 off; 2020 Dec 24,31 off; Sa 10:00-12:00; 2020 Dec 26,2021 Jan 02 off; PH off");
 
         // real-world tests from Osmose that we were handling wrongly
         T("Tu-Fr 11:30-14:30 open, 14:30-18:00 open \"pickup only\", 18:00-22:00 open");
@@ -204,7 +211,14 @@ private Q_SLOTS:
         T2("2020-2021 Mo, Tu-Fr, Sa [-1] 09:00-14:00", "2020-2021 Mo,Tu-Fr,Sa[-1] 09:00-14:00");
         T2("week 1-3 Mo[2], Tu-Fr, Sa [-1] 09:00-14:00", "week 01-03 Mo[2],Tu-Fr,Sa[-1] 09:00-14:00");
         T2("Mo Fr 09:30-12:30 13:30-18:30", "Mo,Fr 09:30-12:30,13:30-18:30");
+        T2("Mo Fr 09:30-12:30, 13:30-18:30 off", "Mo,Fr 09:30-12:30,13:30-18:30 off");
         T2("Mo, We, Fr 06:30-21:30; Tu, Th 09:00-21:30; Sa 09:00-17:00; Su 09:00-14:00", "Mo,We,Fr 06:30-21:30; Tu,Th 09:00-21:30; Sa 09:00-17:00; Su 09:00-14:00");
+        T2("Lunes a sábado, 9:30 AM-5:30 PM", "Mo-Sa 09:30-17:30"); // bug 445784
+
+        // lists of specific days
+        T("2021 Dec 26-28,30-31,2022 Jan 02-03 off");
+        T2("2021 Dec 26-28,Dec 30-31,2022 Jan 02-03 off", "2021 Dec 26-28,30-31,2022 Jan 02-03 off");
+        T2("2021 Dec 22, 26-28, 29, 31, 2022 Jan 02-03 off", "2021 Dec 22,26-28,29,31,2022 Jan 02-03 off");
 
         // technically wrong but often found content in OSM for which we have error recovery
         T2("So", "Su");
@@ -221,9 +235,12 @@ private Q_SLOTS:
         T2("Friday 08:00-12:00", "Fr 08:00-12:00");
         T2("Sat", "Sa");
         T2("december", "Dec");
-        T2("Dec 24,25,26, Jan 1,6 off", "Dec 24,Dec 25,Dec 26,Jan 01,Jan 06 off");
-        T2("Dec 24,25,26 open, Jan 1,6 off", "Dec 24,Dec 25,Dec 26 open, Jan 01,Jan 06 off");
-        T2("07:30-20:00; Jan 03, 13, 23, Feb 03, 13, 23, Mar 03, 13, 23, Apr 03, 13, 23, Jun 03, 13, 23, Jul 03, 13, 23, Aug 03, 13, 23, Sep 03, 13, 23, Oct 03, 13, 23, Nov 03, 13, 23, Dec 03, 13, 23 off", "07:30-20:00; Jan 03,Jan 13,Jan 23,Feb 03,Feb 13,Feb 23,Mar 03,Mar 13,Mar 23,Apr 03,Apr 13,Apr 23,Jun 03,Jun 13,Jun 23,Jul 03,Jul 13,Jul 23,Aug 03,Aug 13,Aug 23,Sep 03,Sep 13,Sep 23,Oct 03,Oct 13,Oct 23,Nov 03,Nov 13,Nov 23,Dec 03,Dec 13,Dec 23 off");
+        T2("Dec 24,25,26, Jan 1,6 off", "Dec 24,25,26,Jan 01,06 off");
+        T2("Dec 24,25,26 open, Jan 1,6 off", "Dec 24,25,26 open, Jan 01,06 off");
+        T2("Dec 6,4", "Dec 06,04");
+        T2("Dec 3,2,1", "Dec 03,02,01");
+        T2("07:30-20:00; Jan 03, 13, 23, Feb 03, 13, 23, Mar 03, 13, 23, Apr 03, 13, 23, Jun 03, 13, 23, Jul 03, 13, 23, Aug 03, 13, 23, Sep 03, 13, 23, Oct 03, 13, 23, Nov 03, 13, 23, Dec 03, 13, 23 off",
+           "07:30-20:00; Jan 03,13,23,Feb 03,13,23,Mar 03,13,23,Apr 03,13,23,Jun 03,13,23,Jul 03,13,23,Aug 03,13,23,Sep 03,13,23,Oct 03,13,23,Nov 03,13,23,Dec 03,13,23 off");
         T2("Apr, May, Oct, Nov, Dec: Mo-Su, 10:00-19:00; Jun-Sep: Mo-Su:10:00-20:00", "Apr,May,Oct,Nov,Dec: Mo-Su, 10:00-19:00; Jun-Sep: Mo-Su 10:00-20:00");
 
         // Tolerance for incorrect casing
@@ -272,6 +289,7 @@ private Q_SLOTS:
         T2("Mo-Th 11:00-20:00 Friday & Saturday 11:00-21:00 Sunday 12:00-19:00", "Mo-Th 11:00-20:00; Fr,Sa 11:00-21:00; Su 12:00-19:00");
         T2("11:30-14:00、16:30-22:00", "11:30-14:00,16:30-22:00");
         T2("Lunedì al Venerdì 08:00-13:00", "Mo-Fr 08:00-13:00");
+        T2("Monday through Friday 09:00 - 17:00", "Mo-Fr 09:00-17:00");
 
         // (mis)use of colon as a small-range selector separator
         T2("Fr: 17:00-19:00", "Fr 17:00-19:00");
@@ -312,6 +330,18 @@ private Q_SLOTS:
         T3("Ma,Me,Je,Ve 8h-12h30, 14h-19h; Sa 8h-12h30, 14h-18h", "Tu,We,Th,Fr 08:00-12:30,14:00-19:00; Sa 08:00-12:30,14:00-18:00", "Tu-Fr 08:00-12:30,14:00-19:00; Sa 08:00-12:30,14:00-18:00");
         T2("Mo-Fr: 10:00-18:30 Uhr Sa: 10:00-13:30 Uhr", "Mo-Fr 10:00-18:30; Sa 10:00-13:30");
         T2("Montag & Dienstag Ruhetag", "Mo,Tu closed");
+        T2("Понедельник - Суббота 09:00 - 21:00", "Mo-Sa 09:00-21:00");
+        T2("Пон-Пт 08:00-20:00, Суб 08:00-19:00", "Mo-Fr 08:00-20:00, Sa 08:00-19:00");
+        T2("Ноябрь-Март", "Nov-Mar");
+        T2("с 08:30 до 23:00", "08:30-23:00");
+        T2("рассвет - сумерки", "dawn-dusk");
+        T2("от рассвета до сумерек", "dawn-dusk");
+        T2("восход - закат", "sunrise-sunset");
+        T2("с восхода пo закат", "sunrise-sunset");
+        T2("от восхода дo заката", "sunrise-sunset");
+        T2("Среда открыто; Пятница закрыто; Суббота неизвестно", "We open; Fr closed; Sa unknown");
+        T2("Вт Выходной", "Tu closed");
+        T2("Mo Cerrado; Tu Abierto, Fr libre", "Mo closed; Tu open, Fr closed");
 
         // recovery from wrong rule separators
         T2("Fr,Sa 10:00-02:00,Su 10:00-20:00", "Fr,Sa 10:00-02:00, Su 10:00-20:00");
@@ -322,6 +352,10 @@ private Q_SLOTS:
         T2("Mo-Sa 12:00-15:00; 18:00-24:00", "Mo-Sa 12:00-15:00,18:00-24:00");
         T2("Mo-Sa 12:00-15:00; Mo-Sa 18:00-24:00", "Mo-Sa 12:00-15:00,18:00-24:00");
         T2("Mo 12:00-15:00; Mo 18:00-24:00", "Mo 12:00-15:00,18:00-24:00");
+        T("Mo-Sa 12:00-15:00 off; 18:00-24:00");
+        T("Mo-Sa 12:00-15:00; Mo-Sa 18:00-24:00 closed");
+        T("Mo 12:00-15:00; Mo 18:00-24:00 \"comment\"");
+        T("Sa 08:00-12:00; 11:30-13:00 off");
 
         // recovery from wrong time selector separators
         T3("Dimanche Fermé Lundi 08:00 – 12:30 14:00 – 19:00 Mardi 08:00 – 12:30 14:00 – 19:00 Mercredi 08:00 – 12:30 14:00 – 19:00 Jeudi 08:00 – 12:30 14:00 – 19:00 Vendredi 08:00 – 12:30 14:00 – 19:00 Samedi 08:00 – 12:30 14:30 – 18:00",
@@ -345,6 +379,20 @@ private Q_SLOTS:
         T3("Sa-Mo 10:00-23:00, Th 10:00-23:00", "Sa-Mo 10:00-23:00, Th 10:00-23:00", "Sa-Mo,Th 10:00-23:00"); // beginDay > endDay
         T3("Sa-Mo 10:00-23:00, Fr 10:00-23:00", "Sa-Mo 10:00-23:00, Fr 10:00-23:00", "Fr-Mo 10:00-23:00"); // beginDay > endDay
         T3("Su-Th 10:00-23:00, Fr-Sa 10:00-23:00", "Su-Th 10:00-23:00, Fr-Sa 10:00-23:00", "Mo-Su 10:00-23:00"); // beginDay > endDay
+        T3("Feb 1-Feb 29 Mo-Su 10:30-20:30; Aug 1-Aug 31 Mo-Fr 10:30-12:00; PH closed",
+           "Feb 01-29 Mo-Su 10:30-20:30; Aug 01-31 Mo-Fr 10:30-12:00; PH closed",
+           "Feb Mo-Su 10:30-20:30; Aug Mo-Fr 10:30-12:00; PH closed");
+
+        // complex or creative 24/7 use
+        T("06:00-01:00 open \"Dining in\" || 24/7 \"Drive-through\"");
+        T2("Friday and Saturday 24/7 Sunday-Thursday 4:00 am to 12:00 am", "Fr,Sa 00:00-24:00; Su-Th 04:00-24:00"); // bug 445962
+        // 24/7 as standalone small range selector not supported yet
+        //T2("Apr-Oct: 24/7; Nov-Mar: Mo-Su 06:00-22:00", "Apr-Oct: 00:00-24:00; Nov-Mar: Mo-Su 06:00-22:00");
+
+        // 4 digit times
+        T2("0600-1800", "06:00-18:00");
+        T2("0700-2000", "07:00-20:00");
+        T2("Tu-Th 8:30-17:30, Fr 8:30-1700", "Tu-Th 08:30-17:30, Fr 08:30-17:00");
 #undef T
 #undef T2
 #undef T3
@@ -359,6 +407,8 @@ private Q_SLOTS:
         QVERIFY(oh.error() != OpeningHours::SyntaxError);
         QCOMPARE(oh.normalizedExpression(), expectedOutput);
         QCOMPARE(oh.simplifiedExpression(), expectedSimplifiedOutput);
+        // verify that simplifiedExpression() doesn't alter `oh`
+        QCOMPARE(oh.normalizedExpression(), expectedOutput);
 
         // verify the expressions we generate are parsed correctly as well
         OpeningHours oh2(oh.normalizedExpression());
@@ -392,14 +442,11 @@ private Q_SLOTS:
         T("12:61");
         T("60p");
 
-        T("Dec 6,4");
         T("Dec 24-Jan 1,6");
-        T("Dec 3,2,1");
         T("Mo, 1:100");
 
         // from https://wiki.openstreetmap.org/wiki/Key:opening_hours#Common_mistakes
         T("7/8-23");
-        T("0600-1800");
         T("07;00-2;00pm");
         T("08.00-16.00, public room till 03.00 a.m");
         T("09:00-21:00 TEL/072(360)3200");
